@@ -3,14 +3,15 @@
     mistune
     ~~~~~~~
 
-    The fastest markdown parser in pure Python, inspired by marked.
+    The fastest markdown parser in pure Python with renderer feature.
 
     :copyright: (c) 2014 by Hsiaoming Yang.
 """
 
 import re
+import inspect
 
-__version__ = '0.3.1'
+__version__ = '0.4'
 __author__ = 'Hsiaoming Yang <me@lepture.com>'
 __all__ = [
     'BlockGrammar', 'BlockLexer',
@@ -119,7 +120,6 @@ class BlockGrammar(object):
         r'(?:\n(?!\2(?:[*+-]|\d+\.) )[^\n]*)*)',
         flags=re.M
     )
-    list_pure_bullet = re.compile(r'(?:[*+-]|\d\.)')
     list_bullet = re.compile(r'^ *(?:[*+-]|\d+\.) +')
     paragraph = re.compile(
         r'^((?:[^\n]+\n?(?!'
@@ -174,8 +174,6 @@ class BlockLexer(object):
     )
 
     def __init__(self, rules=None, **kwargs):
-        self.options = kwargs
-
         self.tokens = []
         self.def_links = {}
         self.def_footnotes = {}
@@ -185,32 +183,32 @@ class BlockLexer(object):
 
         self.rules = rules
 
-    def __call__(self, src, features=None):
-        return self.parse(src, features)
+    def __call__(self, text, features=None):
+        return self.parse(text, features)
 
-    def parse(self, src, features=None):
-        src = src.rstrip('\n')
+    def parse(self, text, features=None):
+        text = text.rstrip('\n')
 
         if not features:
             features = self.default_features
 
-        def manipulate(src):
+        def manipulate(text):
             for key in features:
                 rule = getattr(self.rules, key)
-                m = rule.match(src)
+                m = rule.match(text)
                 if not m:
                     continue
                 getattr(self, 'parse_%s' % key)(m)
                 return m
             return False
 
-        while src:
-            m = manipulate(src)
+        while text:
+            m = manipulate(text)
             if m is not False:
-                src = src[len(m.group(0)):]
+                text = text[len(m.group(0)):]
                 continue
-            if src:
-                raise RuntimeError('Infinite loop at: %s' % src)
+            if text:
+                raise RuntimeError('Infinite loop at: %s' % text)
         return self.tokens
 
     def parse_newline(self, m):
@@ -399,13 +397,8 @@ class BlockLexer(object):
     def parse_block_html(self, m):
         pre = m.group(1) in ['pre', 'script', 'style']
         text = m.group(0)
-        if self.options.get('escape'):
-            text = escape(text)
-            t = 'paragraph'
-        else:
-            t = 'block_html'
         self.tokens.append({
-            'type': t,
+            'type': 'block_html',
             'pre': pre,
             'text': text
         })
@@ -422,7 +415,7 @@ class BlockLexer(object):
 class InlineGrammar(object):
     """Grammars for inline level tokens."""
 
-    escape = re.compile(r'^\\([\\`*{}\[\]()#+\-.!_>~|])')
+    escape = re.compile(r'^\\([\\`*{}\[\]()#+\-.!_>~|])')  # \* \+ \! ....
     tag = re.compile(
         r'^<!--[\s\S]*?-->|'
         r'^<\/?\w+(?:"[^"]*"|'
@@ -453,9 +446,9 @@ class InlineGrammar(object):
         r'|'
         r'^\*((?:\*\*|[\s\S])+?)\*(?!\*)'  # *word*
     )
-    code = re.compile(r'^(`+)\s*([\s\S]*?[^`])\s*\1(?!`)')
+    code = re.compile(r'^(`+)\s*([\s\S]*?[^`])\s*\1(?!`)')  # `code`
     linebreak = re.compile(r'^ {2,}\n(?!\s*$)')
-    strikethrough = re.compile(r'^~~(?=\S)([\s\S]*?\S)~~')
+    strikethrough = re.compile(r'^~~(?=\S)([\s\S]*?\S)~~')  # ~~word~~
     footnote = re.compile(r'^\[\^([^\]]+)\]')
     text = re.compile(r'^[\s\S]+?(?=[\\<!\[_*`~]|https?://| {2,}\n|$)')
 
@@ -480,8 +473,6 @@ class InlineLexer(object):
     ]
 
     def __init__(self, renderer, rules=None, **kwargs):
-        self.options = kwargs
-
         self.renderer = renderer
         self.links = {}
         self.footnotes = {}
@@ -490,27 +481,21 @@ class InlineLexer(object):
         if not rules:
             rules = InlineGrammar()
 
-        if self.options.get('hard_wrap'):
-            rules.hard_wrap()
-
         self.rules = rules
 
         self._in_link = False
         self._in_footnote = False
 
-    def __call__(self, src):
-        return self.output(src)
+    def __call__(self, text):
+        return self.output(text)
 
     def setup(self, links, footnotes):
         self.footnote_index = 0
         self.links = links or {}
         self.footnotes = footnotes or {}
 
-    def output(self, src, features=None):
-        src = src.rstrip('\n')
-        if self.options.get('escape'):
-            src = escape(src)
-
+    def output(self, text, features=None):
+        text = text.rstrip('\n')
         if not features:
             features = list(self.default_features)
 
@@ -519,10 +504,10 @@ class InlineLexer(object):
 
         output = ''
 
-        def manipulate(src):
+        def manipulate(text):
             for key in features:
                 pattern = getattr(self.rules, key)
-                m = pattern.match(src)
+                m = pattern.match(text)
                 if not m:
                     continue
                 self.line_match = m
@@ -532,16 +517,16 @@ class InlineLexer(object):
             return False
 
         self.line_started = False
-        while src:
-            ret = manipulate(src)
+        while text:
+            ret = manipulate(text)
             self.line_started = True
             if ret is not False:
                 m, out = ret
                 output += out
-                src = src[len(m.group(0)):]
+                text = text[len(m.group(0)):]
                 continue
-            if src:
-                raise RuntimeError('Infinite loop at: %s' % src)
+            if text:
+                raise RuntimeError('Infinite loop at: %s' % text)
 
         return output
 
@@ -549,19 +534,17 @@ class InlineLexer(object):
         return m.group(1)
 
     def output_autolink(self, m):
-        data = m.group(1)
+        link = m.group(1)
         if m.group(2) == '@':
             is_email = True
-            link = escape(data)
         else:
             is_email = False
-            link = escape(data)
         return self.renderer.autolink(link, is_email)
 
     def output_url(self, m):
-        link = escape(m.group(1))
+        link = m.group(1)
         if self._in_link:
-            return link
+            return self.renderer.text(link)
         return self.renderer.autolink(link, False)
 
     def output_tag(self, m):
@@ -571,7 +554,7 @@ class InlineLexer(object):
             self._in_link = True
         if lower_text.startswith('</a>'):
             self._in_link = False
-        return text
+        return self.renderer.tag(text)
 
     def output_footnote(self, m):
         key = _keyify(m.group(1))
@@ -602,14 +585,8 @@ class InlineLexer(object):
 
     def _process_link(self, m, link, title=None):
         line = m.group(0)
-
-        if title:
-            title = escape(title, quote=True)
-
         text = m.group(1)
-
         if line[0] == '!':
-            text = escape(text, quote=True)
             return self.renderer.image(link, title, text)
 
         self._in_link = True
@@ -629,7 +606,6 @@ class InlineLexer(object):
 
     def output_code(self, m):
         text = m.group(2)
-        text = escape(text.rstrip(), smart_amp=False)
         return self.renderer.codespan(text)
 
     def output_linebreak(self, m):
@@ -668,16 +644,18 @@ class Renderer(object):
 
         :param text: text content of the blockquote.
         """
-        return '<blockquote>%s\n</blockquote>' % text
+        return '<blockquote>%s</blockquote>\n' % text
 
     def block_html(self, html):
         """Rendering block level pure html content.
 
         :param html: text content of the html snippet.
         """
-        if self.options.get('skip_style'):
-            if html.lower().startswith('<style'):
-                return ''
+        if self.options.get('skip_style') and \
+           html.lower().startswith('<style'):
+            return ''
+        if self.options.get('escape'):
+            return escape(html)
         return html
 
     def header(self, text, level, raw=None):
@@ -769,6 +747,7 @@ class Renderer(object):
 
         :param text: text content for inline code.
         """
+        text = escape(text.rstrip(), smart_amp=False)
         return '<code>%s</code>' % text
 
     def linebreak(self):
@@ -797,7 +776,7 @@ class Renderer(object):
         :param link: link content or email address.
         :param is_email: whether this is an email or not.
         """
-        text = link
+        text = link = escape(link)
         if is_email:
             link = 'mailto:%s' % link
         return '<a href="%s">%s</a>' % (link, text)
@@ -809,11 +788,11 @@ class Renderer(object):
         :param title: title content for `title` attribute.
         :param text: text content for description.
         """
-        if 'javascript:' in link:
-            # for safety
-            return ''
+        if link.startswith('javascript:'):
+            link = ''
         if not title:
             return '<a href="%s">%s</a>' % (link, text)
+        title = escape(title, quote=True)
         return '<a href="%s" title="%s">%s</a>' % (link, title, text)
 
     def image(self, src, title, text):
@@ -823,20 +802,26 @@ class Renderer(object):
         :param title: title text of the image.
         :param text: alt text of the image.
         """
-        if not title:
-            html = '<img src="%s" alt="%s"' % (src, text)
-        else:
+        if src.startswith('javascript:'):
+            src = ''
+        text = escape(text, quote=True)
+        if title:
+            title = escape(title, quote=True)
             html = '<img src="%s" alt="%s" title="%s"' % (src, text, title)
+        else:
+            html = '<img src="%s" alt="%s"' % (src, text)
         if self.options.get('use_xhtml'):
             return '%s />' % html
         return '%s>' % html
 
-    def raw_html(self, html):
-        """Rendering span level html snippet.
+    def tag(self, html):
+        """Rendering span level html tag.
 
-        :param html: html snippet.
+        :param html: html tag snippet.
         """
         if self.options.get('skip_html'):
+            return ''
+        if self.options.get('escape'):
             return escape(html)
         return html
 
@@ -884,26 +869,26 @@ class Markdown(object):
     :param renderer: An instance of ``Renderer``.
     """
 
-    def __init__(self, renderer=None, **kwargs):
-        inline = kwargs.pop('inline', None)
-        block = kwargs.pop('block', None)
-
+    def __init__(self, renderer=None, inline=None, block=None, **kwargs):
         if not renderer:
-            renderer = Renderer()
+            renderer = Renderer(**kwargs)
 
         self.renderer = renderer
-        self.options = kwargs
 
-        if inline and callable(inline):
-            # assume it is a class
+        if inline and inspect.isclass(inline):
             inline = inline(renderer, **kwargs)
-        if block and callable(block):
-            # assume it is a class
+        if block and inspect.isclass(block):
             block = block(**kwargs)
 
-        self.inline = inline or InlineLexer(renderer, **kwargs)
-        self.block = block or BlockLexer(**kwargs)
+        if inline:
+            self.inline = inline
+        else:
+            rules = InlineGrammar()
+            if kwargs.get('hard_wrap'):
+                rules.hard_wrap()
+            self.inline = InlineLexer(renderer, rules=rules)
 
+        self.block = block or BlockLexer(BlockGrammar())
         self.footnotes = []
         self.tokens = []
 
@@ -917,8 +902,8 @@ class Markdown(object):
         """
         return self.parse(text)
 
-    def parse(self, src):
-        out = self.output(src)
+    def parse(self, text):
+        out = self.output(preprocessing(text))
         keys = self.block.def_footnotes
 
         # reset block
@@ -947,10 +932,8 @@ class Markdown(object):
         out += self.renderer.footnotes(body)
         return out
 
-    def output(self, src, features=None):
-        src = preprocessing(src)
-
-        self.tokens = self.block(src, features)
+    def output(self, text, features=None):
+        self.tokens = self.block(text, features)
         self.tokens.reverse()
 
         self.inline.setup(self.block.def_links, self.block.def_footnotes)
@@ -1069,8 +1052,6 @@ class Markdown(object):
 
     def parse_block_html(self):
         text = self.token['text']
-        if 'pre' not in self.token or not self.token['pre']:
-            text = self.inline(text)
         return self.renderer.block_html(text)
 
     def parse_paragraph(self):
